import re

_VERSION_PATTERN = re.compile(r"^(\d+)\.(\d+)")


def parse_spark_version(version: str) -> tuple[int, int] | None:
    """Parse Spark version string to (major, minor) tuple. Returns None if unparseable."""
    if not version:
        return None
    match = _VERSION_PATTERN.match(version.strip())
    if not match:
        return None
    return int(match.group(1)), int(match.group(2))



from __future__ import annotations

import uuid
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from opentelemetry.context import Context
    from opentelemetry.trace import Tracer

TRACE_ID_HEADER = "X-Trace-ID"

__all__ = [
    "TRACE_ID_HEADER",
    "TracingConfig",
    "add_otel_trace_context",
    "build_trace_context_vars",
    "configure_tracing",
    "extract_fallback_trace_id",
    "extract_trace_context",
    "get_tracer",
    "inject_correlation_context",
    "inject_trace_context",
]


class TracingConfig:
    _service_name: str = "spark-advisor"

    @classmethod
    def configure(cls, service_name: str, endpoint: str, *, enabled: bool = False) -> None:
        cls._service_name = service_name

        if not enabled:
            return

        from opentelemetry import trace
        from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
        from opentelemetry.sdk.resources import SERVICE_NAME, Resource
        from opentelemetry.sdk.trace import TracerProvider
        from opentelemetry.sdk.trace.export import BatchSpanProcessor

        resource = Resource.create({SERVICE_NAME: service_name})
        provider = TracerProvider(resource=resource)
        provider.add_span_processor(BatchSpanProcessor(OTLPSpanExporter(endpoint=endpoint)))
        trace.set_tracer_provider(provider)

    @classmethod
    def get_tracer(cls) -> Tracer:
        from opentelemetry import trace

        return trace.get_tracer(cls._service_name)


def configure_tracing(service_name: str, endpoint: str, *, enabled: bool = False) -> None:
    TracingConfig.configure(service_name, endpoint, enabled=enabled)


def get_tracer() -> Tracer:
    return TracingConfig.get_tracer()


def inject_trace_context(headers: dict[str, str]) -> dict[str, str]:
    from opentelemetry import context
    from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator

    TraceContextTextMapPropagator().inject(headers, context=context.get_current())
    return headers


def extract_trace_context(headers: dict[str, str] | None) -> Context:
    from opentelemetry import context
    from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator

    if not headers:
        return context.get_current()
    return TraceContextTextMapPropagator().extract(carrier=headers)


def add_otel_trace_context(
    logger: Any, method: str, event_dict: dict[str, Any],
) -> dict[str, Any]:
    from opentelemetry import trace

    span = trace.get_current_span()
    ctx = span.get_span_context()
    if ctx.is_valid:
        event_dict["trace_id"] = format(ctx.trace_id, "032x")
        event_dict["span_id"] = format(ctx.span_id, "016x")
    return event_dict


def inject_correlation_context(headers: dict[str, str]) -> dict[str, str]:
    headers = inject_trace_context(headers)
    if "traceparent" not in headers and TRACE_ID_HEADER not in headers:
        headers[TRACE_ID_HEADER] = str(uuid.uuid4())
    return headers


def extract_fallback_trace_id(headers: dict[str, str] | None) -> str:
    from opentelemetry import trace

    span = trace.get_current_span()
    if span.get_span_context().is_valid:
        return ""
    return (headers or {}).get(TRACE_ID_HEADER, "")


def build_trace_context_vars() -> dict[str, str]:
    from opentelemetry import trace

    span = trace.get_current_span()
    if span.get_span_context().is_valid:
        return {}
    return {"trace_id": str(uuid.uuid4())}



from __future__ import annotations

from opentelemetry import context, trace
from opentelemetry.sdk.resources import SERVICE_NAME, Resource
from opentelemetry.sdk.trace import TracerProvider

from spark_advisor_models.tracing import (
    TRACE_ID_HEADER,
    TracingConfig,
    add_otel_trace_context,
    build_trace_context_vars,
    configure_tracing,
    extract_fallback_trace_id,
    extract_trace_context,
    get_tracer,
    inject_correlation_context,
    inject_trace_context,
)


def _fresh_provider() -> TracerProvider:
    return TracerProvider(resource=Resource.create({SERVICE_NAME: "test"}))


class TestTracingConfig:
    def setup_method(self) -> None:
        TracingConfig._service_name = "spark-advisor"
        trace.set_tracer_provider(_fresh_provider())

    def test_configure_sets_service_name(self) -> None:
        configure_tracing("my-service", "http://localhost:4317", enabled=False)
        assert TracingConfig._service_name == "my-service"

    def test_disabled_does_not_set_provider(self) -> None:
        configure_tracing("test", "http://localhost:4317", enabled=False)
        tracer = get_tracer()
        assert tracer is not None

    def test_get_tracer_returns_tracer_with_service_name(self) -> None:
        TracingConfig._service_name = "my-service"
        tracer = get_tracer()
        assert tracer is not None


class TestTraceContextPropagation:
    def setup_method(self) -> None:
        trace.set_tracer_provider(_fresh_provider())

    def test_inject_extract_roundtrip(self) -> None:
        tracer = trace.get_tracer("test")
        with tracer.start_as_current_span("parent") as span:
            headers: dict[str, str] = {}
            inject_trace_context(headers)
            assert "traceparent" in headers

            extracted_ctx = extract_trace_context(headers)
            extracted_span = trace.get_current_span(extracted_ctx)
            assert extracted_span.get_span_context().trace_id == span.get_span_context().trace_id

    def test_extract_none_headers_returns_context(self) -> None:
        ctx = extract_trace_context(None)
        assert ctx is not None

    def test_extract_empty_headers_returns_context(self) -> None:
        ctx = extract_trace_context({})
        assert ctx is not None

    def test_inject_without_active_span(self) -> None:
        context.attach(context.Context())
        headers: dict[str, str] = {}
        result = inject_trace_context(headers)
        assert result is headers


class TestOtelTraceContextProcessor:
    def setup_method(self) -> None:
        trace.set_tracer_provider(_fresh_provider())

    def test_adds_trace_id_and_span_id_with_active_span(self) -> None:
        tracer = trace.get_tracer("test")
        with tracer.start_as_current_span("test-span") as span:
            event_dict: dict[str, object] = {"event": "hello"}
            result = add_otel_trace_context(None, "info", event_dict)
            assert "trace_id" in result
            assert "span_id" in result
            assert result["trace_id"] == format(span.get_span_context().trace_id, "032x")

    def test_no_trace_fields_without_active_span(self) -> None:
        context.attach(context.Context())
        event_dict: dict[str, object] = {"event": "hello"}
        result = add_otel_trace_context(None, "info", event_dict)
        assert "trace_id" not in result
        assert "span_id" not in result


class TestInjectCorrelationContext:
    def setup_method(self) -> None:
        trace.set_tracer_provider(_fresh_provider())

    def test_with_otel_injects_traceparent_not_fallback(self) -> None:
        tracer = trace.get_tracer("test")
        with tracer.start_as_current_span("parent"):
            headers: dict[str, str] = {}
            result = inject_correlation_context(headers)
            assert "traceparent" in result
            assert TRACE_ID_HEADER not in result

    def test_without_otel_injects_fallback_uuid(self) -> None:
        context.attach(context.Context())
        headers: dict[str, str] = {}
        result = inject_correlation_context(headers)
        assert "traceparent" not in result
        assert TRACE_ID_HEADER in result
        import uuid

        uuid.UUID(result[TRACE_ID_HEADER])

    def test_does_not_overwrite_existing_trace_id_header(self) -> None:
        context.attach(context.Context())
        headers = {TRACE_ID_HEADER: "existing-uuid"}
        result = inject_correlation_context(headers)
        assert result[TRACE_ID_HEADER] == "existing-uuid"


class TestExtractFallbackTraceId:
    def setup_method(self) -> None:
        trace.set_tracer_provider(_fresh_provider())

    def test_returns_empty_when_otel_active(self) -> None:
        tracer = trace.get_tracer("test")
        with tracer.start_as_current_span("parent"):
            headers: dict[str, str] = {}
            inject_trace_context(headers)

        extracted_ctx = extract_trace_context(headers)
        context.attach(extracted_ctx)
        assert extract_fallback_trace_id(headers) == ""

    def test_returns_header_value_without_otel(self) -> None:
        context.attach(context.Context())
        headers = {TRACE_ID_HEADER: "my-uuid-123"}
        assert extract_fallback_trace_id(headers) == "my-uuid-123"

    def test_returns_empty_when_no_header(self) -> None:
        context.attach(context.Context())
        assert extract_fallback_trace_id(None) == ""
        assert extract_fallback_trace_id({}) == ""


class TestBuildTraceContextVars:
    def setup_method(self) -> None:
        trace.set_tracer_provider(_fresh_provider())

    def test_empty_with_active_otel_span(self) -> None:
        tracer = trace.get_tracer("test")
        with tracer.start_as_current_span("parent"):
            assert build_trace_context_vars() == {}

    def test_uuid_without_otel_span(self) -> None:
        context.attach(context.Context())
        result = build_trace_context_vars()
        assert "trace_id" in result
        import uuid

        uuid.UUID(result["trace_id"])



from spark_advisor_models.util.bytes import format_bytes, parse_memory_string
from spark_advisor_models.util.spark import parse_spark_version
from spark_advisor_models.util.stats import median_value, percentile_value, quantiles_5

__all__ = [
    "format_bytes",
    "median_value",
    "parse_memory_string",
    "parse_spark_version",
    "percentile_value",
    "quantiles_5",
]



import re

_MEMORY_PATTERN = re.compile(r"^(\d+(?:\.\d+)?)\s*([kmgt])?b?$", re.IGNORECASE)
_SUFFIX_TO_MB: dict[str, int] = {"m": 1, "g": 1024, "t": 1024 * 1024}


def parse_memory_string(value: str) -> int:
    """Parse Spark memory string (e.g. '1g', '512m', '2048') to megabytes."""
    if not value or not value.strip():
        return 0
    match = _MEMORY_PATTERN.match(value.strip())
    if not match:
        return 0
    number = float(match.group(1))
    suffix = (match.group(2) or "m").lower()
    if suffix == "k":
        return max(1, int(number / 1024))
    return int(number * _SUFFIX_TO_MB.get(suffix, 1))


def format_bytes(n: int | float) -> str:
    value = float(n)
    for unit in ("B", "KB", "MB", "GB", "TB"):
        if abs(value) < 1024:
            return f"{value:.1f} {unit}"
        value /= 1024
    return f"{value:.1f} PB"



from spark_advisor_models.config import AiSettings, Thresholds

DEFAULT_MODEL = "claude-sonnet-4-6"
DEFAULT_THRESHOLDS = Thresholds()
DEFAULT_AI_SETTINGS = AiSettings()

NATS_FETCH_JOB_SUBJECT = "job.fetch"
NATS_ANALYSIS_RUN_SUBJECT = "analysis.run"
NATS_ANALYSIS_RUN_AGENT_SUBJECT = "analysis.run.agent"
NATS_ANALYSIS_RESULT_SUBJECT = "analysis.result"
NATS_APPLICATIONS_LIST_SUBJECT = "apps.list"
NATS_ANALYSIS_SUBMIT_SUBJECT = "analysis.submit"
NATS_STORAGE_FETCH_HDFS_SUBJECT = "storage.fetch.hdfs"
NATS_STORAGE_FETCH_S3_SUBJECT = "storage.fetch.s3"
NATS_STORAGE_FETCH_GCS_SUBJECT = "storage.fetch.gcs"
NATS_FETCH_JOB_K8S_SUBJECT = "job.fetch.k8s"

DEFAULT_EVENT_LOGS_DIR = "/spark-events"



from spark_advisor_models.model.spark_config import SparkConfig


class TestSparkConfigDriverProperties:
    def test_driver_memory_default(self) -> None:
        config = SparkConfig(raw={})
        assert config.driver_memory == "1g"

    def test_driver_memory_explicit(self) -> None:
        config = SparkConfig(raw={"spark.driver.memory": "4g"})
        assert config.driver_memory == "4g"

    def test_driver_memory_overhead_default(self) -> None:
        config = SparkConfig(raw={})
        assert config.driver_memory_overhead == ""

    def test_driver_memory_overhead_explicit(self) -> None:
        config = SparkConfig(raw={"spark.driver.memoryOverhead": "512m"})
        assert config.driver_memory_overhead == "512m"

    def test_has_aqe_config_key_present(self) -> None:
        config = SparkConfig(raw={"spark.sql.adaptive.enabled": "true"})
        assert config.has_explicit_aqe_config is True

    def test_has_aqe_config_key_absent(self) -> None:
        config = SparkConfig(raw={})
        assert config.has_explicit_aqe_config is False



from spark_advisor_models.util.bytes import parse_memory_string


class TestParseMemoryString:
    def test_gigabytes_suffix(self) -> None:
        assert parse_memory_string("1g") == 1024

    def test_megabytes_suffix(self) -> None:
        assert parse_memory_string("512m") == 512

    def test_kilobytes_suffix(self) -> None:
        assert parse_memory_string("1024k") == 1

    def test_sub_kb_rounds_up_to_1(self) -> None:
        assert parse_memory_string("512k") == 1

    def test_terabytes_suffix(self) -> None:
        assert parse_memory_string("1t") == 1024 * 1024

    def test_uppercase_suffix(self) -> None:
        assert parse_memory_string("2G") == 2048

    def test_bare_number_treated_as_mb(self) -> None:
        assert parse_memory_string("2048") == 2048

    def test_empty_string_returns_zero(self) -> None:
        assert parse_memory_string("") == 0

    def test_invalid_string_returns_zero(self) -> None:
        assert parse_memory_string("abc") == 0



from __future__ import annotations

import logging
from contextlib import asynccontextmanager
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from collections.abc import AsyncIterator

import structlog

from spark_advisor_models.tracing import add_otel_trace_context, get_tracer

__all__ = [
    "bind_nats_context",
    "configure_logging",
    "nats_handler_context",
]

_TRANSIENT_CONTEXT_KEYS = ("trace_id", "app_id", "task_id")


def configure_logging(service: str, log_level: str, *, json_output: bool = True) -> None:
    shared_processors: list[structlog.types.Processor] = [
        structlog.contextvars.merge_contextvars,
        add_otel_trace_context,  # type: ignore[list-item]
        structlog.stdlib.add_log_level,
        structlog.stdlib.add_logger_name,
        structlog.processors.TimeStamper(fmt="iso"),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
    ]

    renderer = structlog.processors.JSONRenderer() if json_output else structlog.dev.ConsoleRenderer()

    structlog.configure(
        processors=[
            *shared_processors,
            structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
        ],
        logger_factory=structlog.stdlib.LoggerFactory(),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )

    formatter = structlog.stdlib.ProcessorFormatter(
        processors=[
            structlog.stdlib.ProcessorFormatter.remove_processors_meta,
            renderer,
        ],
        foreign_pre_chain=shared_processors,
    )

    handler = logging.StreamHandler()
    handler.setFormatter(formatter)

    root = logging.getLogger()
    root.handlers = [
        h for h in root.handlers
        if not isinstance(h.formatter, structlog.stdlib.ProcessorFormatter)
    ]
    root.addHandler(handler)
    root.setLevel(log_level.upper())

    structlog.contextvars.bind_contextvars(service=service)


def bind_nats_context(headers: dict[str, str] | None, **extra: str) -> None:
    from opentelemetry import context

    from spark_advisor_models.tracing import extract_fallback_trace_id, extract_trace_context

    otel_ctx = extract_trace_context(headers)
    context.attach(otel_ctx)

    fallback_trace_id = extract_fallback_trace_id(headers)

    structlog.contextvars.unbind_contextvars(*_TRANSIENT_CONTEXT_KEYS)

    if fallback_trace_id:
        structlog.contextvars.bind_contextvars(trace_id=fallback_trace_id, **extra)
    else:
        structlog.contextvars.bind_contextvars(**extra)


@asynccontextmanager
async def nats_handler_context(
    msg_headers: dict[str, str] | None,
    span_name: str,
    span_attributes: dict[str, Any],
    **log_context: str,
) -> AsyncIterator[None]:
    bind_nats_context(msg_headers, **log_context)
    with get_tracer().start_as_current_span(span_name, attributes=span_attributes):
        yield



from __future__ import annotations

import logging

import pytest
import structlog

from spark_advisor_models.logging import bind_nats_context, configure_logging, nats_handler_context


class TestConfigureLogging:
    def setup_method(self) -> None:
        structlog.reset_defaults()
        structlog.contextvars.clear_contextvars()
        root = logging.getLogger()
        root.handlers.clear()

    def test_configures_root_handler(self) -> None:
        configure_logging("test-service", "INFO")
        root = logging.getLogger()
        structlog_handlers = [
            h for h in root.handlers
            if isinstance(h.formatter, structlog.stdlib.ProcessorFormatter)
        ]
        assert len(structlog_handlers) == 1
        assert root.level == logging.INFO

    def test_json_renderer_produces_valid_json(self, capsys: object) -> None:
        configure_logging("test-service", "INFO", json_output=True)
        logger = structlog.stdlib.get_logger("test")
        logger.info("hello")
        import sys

        sys.stderr.flush()

    def test_console_renderer_when_json_disabled(self) -> None:
        configure_logging("test-service", "DEBUG", json_output=False)
        root = logging.getLogger()
        formatter = root.handlers[0].formatter
        assert formatter is not None

    def test_log_level_respected(self) -> None:
        configure_logging("test-service", "WARNING")
        root = logging.getLogger()
        assert root.level == logging.WARNING

    def test_log_level_case_insensitive(self) -> None:
        configure_logging("test-service", "debug")
        root = logging.getLogger()
        assert root.level == logging.DEBUG

    def test_service_bound_to_context(self) -> None:
        configure_logging("my-service", "INFO")
        ctx = structlog.contextvars.get_contextvars()
        assert ctx["service"] == "my-service"

    def test_foreign_logs_structured(self, capsys: object) -> None:
        configure_logging("test-service", "INFO", json_output=True)
        stdlib_logger = logging.getLogger("some.third.party")
        stdlib_logger.info("foreign log")

    def test_structlog_get_logger_works(self) -> None:
        configure_logging("test-service", "INFO")
        logger = structlog.stdlib.get_logger("mymodule")
        assert logger is not None

    def test_idempotent_reconfiguration(self) -> None:
        configure_logging("svc1", "INFO")
        configure_logging("svc2", "DEBUG")
        root = logging.getLogger()
        structlog_handlers = [
            h for h in root.handlers
            if isinstance(h.formatter, structlog.stdlib.ProcessorFormatter)
        ]
        assert len(structlog_handlers) == 1
        assert root.level == logging.DEBUG

    def test_preserves_non_structlog_handlers(self) -> None:
        root = logging.getLogger()
        pre_count = len(root.handlers)
        external_handler = logging.StreamHandler()
        root.addHandler(external_handler)
        configure_logging("test-service", "INFO")
        assert external_handler in root.handlers
        structlog_handlers = [
            h for h in root.handlers
            if isinstance(h.formatter, structlog.stdlib.ProcessorFormatter)
        ]
        assert len(structlog_handlers) == 1
        assert len(root.handlers) == pre_count + 2


class TestBindNatsContext:
    def setup_method(self) -> None:
        structlog.contextvars.clear_contextvars()

    def test_binds_service_from_kwargs(self) -> None:
        bind_nats_context(None, service="analyzer")
        ctx = structlog.contextvars.get_contextvars()
        assert ctx["service"] == "analyzer"

    def test_binds_extra_kwargs(self) -> None:
        bind_nats_context(None, app_id="app-42", service="analyzer")
        ctx = structlog.contextvars.get_contextvars()
        assert ctx["app_id"] == "app-42"
        assert ctx["service"] == "analyzer"

    def test_unbinds_transient_keys_preserving_persistent(self) -> None:
        structlog.contextvars.bind_contextvars(service="my-service", old_key="preserved")
        bind_nats_context(None, app_id="app-1")
        ctx = structlog.contextvars.get_contextvars()
        assert ctx.get("service") == "my-service"
        assert ctx.get("old_key") == "preserved"
        assert ctx.get("app_id") == "app-1"

    def test_clears_transient_keys_from_previous_call(self) -> None:
        structlog.contextvars.bind_contextvars(service="svc")
        bind_nats_context(None, app_id="app-1", service="svc")
        bind_nats_context(None, service="svc")
        ctx = structlog.contextvars.get_contextvars()
        assert "app_id" not in ctx

    def test_attaches_otel_context_from_traceparent(self) -> None:
        from opentelemetry import trace
        from opentelemetry.sdk.resources import SERVICE_NAME, Resource
        from opentelemetry.sdk.trace import TracerProvider

        provider = TracerProvider(resource=Resource.create({SERVICE_NAME: "test"}))
        trace.set_tracer_provider(provider)
        tracer = trace.get_tracer("test")

        with tracer.start_as_current_span("parent") as span:
            from spark_advisor_models.tracing import inject_trace_context

            headers: dict[str, str] = {}
            inject_trace_context(headers)

        bind_nats_context(headers, service="analyzer")
        current_span = trace.get_current_span()
        assert current_span.get_span_context().trace_id == span.get_span_context().trace_id

    def test_no_fallback_trace_id_when_otel_active(self) -> None:
        from opentelemetry import trace
        from opentelemetry.sdk.resources import SERVICE_NAME, Resource
        from opentelemetry.sdk.trace import TracerProvider

        provider = TracerProvider(resource=Resource.create({SERVICE_NAME: "test"}))
        trace.set_tracer_provider(provider)
        tracer = trace.get_tracer("test")

        with tracer.start_as_current_span("parent"):
            from spark_advisor_models.tracing import inject_trace_context

            headers: dict[str, str] = {}
            inject_trace_context(headers)

        bind_nats_context(headers, service="analyzer")
        ctx = structlog.contextvars.get_contextvars()
        assert "trace_id" not in ctx

    def test_trace_id_fallback_to_header(self) -> None:
        from spark_advisor_models.tracing import TRACE_ID_HEADER

        bind_nats_context({TRACE_ID_HEADER: "uuid-123"}, service="analyzer")
        ctx = structlog.contextvars.get_contextvars()
        assert ctx["trace_id"] == "uuid-123"

    def test_no_trace_id_when_no_otel_no_header(self) -> None:
        bind_nats_context(None, service="analyzer")
        ctx = structlog.contextvars.get_contextvars()
        assert "trace_id" not in ctx


class TestNatsHandlerContext:
    def setup_method(self) -> None:
        structlog.contextvars.clear_contextvars()

    @pytest.mark.asyncio
    async def test_binds_context_and_creates_span(self) -> None:
        from opentelemetry import trace
        from opentelemetry.sdk.resources import SERVICE_NAME, Resource
        from opentelemetry.sdk.trace import TracerProvider

        provider = TracerProvider(resource=Resource.create({SERVICE_NAME: "test"}))
        trace.set_tracer_provider(provider)

        async with nats_handler_context(
            None, "test.span", {"key": "val"}, service="my-svc",
        ):
            span = trace.get_current_span()
            assert span.get_span_context().is_valid
            ctx = structlog.contextvars.get_contextvars()
            assert ctx["service"] == "my-svc"

    @pytest.mark.asyncio
    async def test_span_ends_after_context_exit(self) -> None:
        from opentelemetry import context, trace
        from opentelemetry.sdk.resources import SERVICE_NAME, Resource
        from opentelemetry.sdk.trace import TracerProvider

        provider = TracerProvider(resource=Resource.create({SERVICE_NAME: "test"}))
        trace.set_tracer_provider(provider)
        context.attach(context.Context())

        async with nats_handler_context(
            None, "test.span", {}, service="svc",
        ):
            pass

        span = trace.get_current_span()
        assert not span.get_span_context().is_valid



from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class Attempt(BaseModel):
    model_config = ConfigDict(frozen=True, alias_generator=to_camel, populate_by_name=True)

    attempt_id: str | None = None
    start_time: str = ""
    end_time: str = ""
    last_updated: str = ""
    duration: int = 0
    spark_user: str = ""
    completed: bool = False
    app_spark_version: str = ""
    log_path: str = ""
    start_time_epoch: int = 0
    end_time_epoch: int = 0
    last_updated_epoch: int = 0


class ApplicationSummary(BaseModel):
    model_config = ConfigDict(frozen=True, alias_generator=to_camel, populate_by_name=True)

    id: str
    name: str = ""
    attempts: list[Attempt] = []
    driver_host: str = ""

    @property
    def latest_attempt(self) -> Attempt | None:
        return self.attempts[-1] if self.attempts else None



from spark_advisor_models.util.spark import parse_spark_version


class TestParseSparkVersion:
    def test_standard_version(self) -> None:
        assert parse_spark_version("3.2.1") == (3, 2)

    def test_major_minor_only(self) -> None:
        assert parse_spark_version("3.4") == (3, 4)

    def test_vendor_suffix(self) -> None:
        assert parse_spark_version("3.5.0-amzn-0") == (3, 5)

    def test_spark2(self) -> None:
        assert parse_spark_version("2.4.8") == (2, 4)

    def test_empty_string(self) -> None:
        assert parse_spark_version("") is None

    def test_unparseable(self) -> None:
        assert parse_spark_version("custom-build") is None

    def test_single_number(self) -> None:
        assert parse_spark_version("3") is None
